;-------------------------------------------------| Febr. 2, 2000
;           "Othello v1.0"                        |
;Description: TI82 assembler game for Ash 3.0     |
;-------------------------------------------------|
;-----------------------------------------|
;Author: Jakob Selbing, JSS (C) 1997-2000 |
;  Comments, bugs, tips, anything on      |
;  your mind:                             |
;  jakob_selbing@hotmail.com              |
;-----------------------------------------|
;
;The game
;----------------------------------------------------------------
;The game Othello is very simple. The game is a 2-player game
;but in this version you play against TI82. 2 players compete in 
;filling a 64-squares board with their markers; one player has black
;color, the other has white. You start with 2 markers each in the 
;middle of the board. When you place your marker on the board, you try
;to capture the opponents markers between that marker and other markers
;of your color that are already on the board. Captured markers will
;change color. Here's an example:
;
;
;(1) After 2 moves each		(2) It's X's turn and he
; it might look like this: 	    places his marker here:
;	
;	0				0
;	0XX				0XX
;	00				00
;	 0				 0
;					 X  <--
;
;
;(3) The two 0's are captured
;and will then change color:
;
;	0
;	0XX
;	0X <-
;	 X <-
;	 X
;
;Captured markers in all directions will change. 
;Markers can also be captured diagonally. You cannot place markers
;where you won't capture any other markers, unless there is only
;such places. In that case you must place your marker next to other
;markers. The game's over when the board is full, and the player with
;most markers wins.
;
;Controls
;-------------
;Del	 - reset board/choose white
;Arrows  - move cursor
;Enter   - place marker
;Mode    - quit
;
;
;Miscellaneous
;----------------------------------
;Startup:
;To reset the board press Del when you're prompterd to do so. If
;you want to be white color, press Del when prompted. Any other key
;will make TI82 white color.
;
;Scanning:
;You'll see some numbers in the right lower corner of the screen
;when you place markers, or when the TI82 places markers. This shows
;that the computer tests all possible moves, and also makes sure you 
;don't make an illegal move.
;
;A.I:
;The TI82 only places its marker to get the highest capture rate possible.
;It doesn't know of tactics; I figured it'd be pretty damn hard to program.
;
;Tip:
;Get corners or edges!!!
;
;ENJOY!!!!
;-----------------------------------------------------
;The compiled file is about 2251 bytes...

#include "ti82.h"
#include "keys.inc"


eval_position   =TEXT_MEM
eval_wins       =TEXT_MEM+1
best_position   =TEXT_MEM+2
best_wins       =TEXT_MEM+3

slength         =TEXT_MEM+5
drawtype        =TEXT_MEM+6
tempx           =TEXT_MEM+7
tempy           =TEXT_MEM+8
blacks          =TEXT_MEM+9
whites          =TEXT_MEM+10

cursor_pos      =TEXT_MEM+11

black   =%01000000
white   =%10000000
unused  =%11000000

delay_con = $0001

mark_info_position_player =GRAPH_MEM+104
mark_info_position_ti82 =GRAPH_MEM+320

score_player_position   =$084C
score_ti82_position     =$1A4C

eval_pos_con    =$2c41
eval_val_con    =$2c51

        .org START_ADDR
        .db "Othello by JSS",0


        ROM_CALL(CLEARLCD)
        ld hl,GRAPH_MEM
        ld de,GRAPH_MEM+1     
        ld bc,767
        ld (hl),$00
        ldir

        ld a,31
        ld (cursor_pos),a

        call restore_game_query

        call display_main_screen
        call inv_cursor


keyloop
        call GET_KEY
        cp G_MODE
        ret z
        cp G_UP
        jr z,move_cursor_up
        cp G_DOWN
        jr z,move_cursor_down
        cp G_LEFT
        jr z,move_cursor_left
        cp G_RIGTH
        jr z,move_cursor_right
        cp G_ENTER
        jr z,place_player_mark
        jr keyloop


move_cursor_up
        call inv_cursor
        ld a,(cursor_pos)
        sub 8
        jr nc,not_too_high
        add a,8
not_too_high
        ld (cursor_pos),a
        call inv_cursor
        jr keyloop

move_cursor_down
        call inv_cursor
        ld a,(cursor_pos)
        add a,8
        cp 64
        jr c,not_too_low
        sub 8
not_too_low
        ld (cursor_pos),a
        call inv_cursor
        jr keyloop

move_cursor_left
        ld c,255
        ld e,%00000111
        jp move_cursor_sideways

move_cursor_right
        ld c,1
        ld e,0
;        jp move_cursor_sideways

move_cursor_sideways
        call inv_cursor
        ld a,(cursor_pos)
        add a,c
        ld c,a
        and %00000111
        cp e
        jr nz,not_on_edge
        ld a,(cursor_pos)
        ld c,a
not_on_edge
        ld a,c
        ld (cursor_pos),a
        call inv_cursor
        jp keyloop


place_player_mark
        ld a,(cursor_pos)
        call getcur_value
        or a
        jr nz,keyloop_branch
        call check_legal_move
        ld a,c
        or a
        jr z,keyloop_branch
        call get_player_color
        ld d,b
        ld a,(cursor_pos)
        call evaluate_cur_pos
        ld a,e
        or a
        jr nz,no_cheating
        call get_player_color
        ld d,b
        call get_best_position
        ld a,(best_wins)
        or a
        jr nz,keyloop_branch

no_cheating
        call get_player_color
        ld a,(cursor_pos)
        call putval_at_cursor
        call inv_cursor
        ld a,(cursor_pos)
        ld hl,GRAPH_MEM-96
        call put_de_8_lines_at_a
        call display_graph_at_cursor
        call get_player_color
        ld d,b
        call change_marks_next_cursor
        call display_main_screen
        jp computer_moves

keyloop_branch
        jp keyloop

get_player_color
        ld b,white
        ld de,sprite_white_mark
        ld a,(computer_color)
        cp black
        jr z,player_places_white
        ld b,black
        ld de,sprite_black_mark
player_places_white
        ret

check_legal_move
        ld c,0
        ld a,(cursor_pos)
        and %00000111
        jr z,test_n
        ld a,(cursor_pos)
        sub 9
        jr c,test_n
        call getcur_value
        or a
        jr z,test_n
        ld c,1
test_n
        ld a,(cursor_pos)
        sub 8
        jr c,test_ne
        call getcur_value
        or a
        jr z,test_ne
        ld c,1
test_ne
        ld a,(cursor_pos)
        sub 7
        jr c,test_w
        and %00000111
        jr z,test_w
        ld a,(cursor_pos)
        sub 7
        call getcur_value
        or a
        jr z,test_w
        ld c,1
test_w
        ld a,(cursor_pos)
        and %00000111
        jr z,test_e
        ld a,(cursor_pos)
        dec a
        call getcur_value
        or a
        jr z,test_e
        ld c,1
test_e
        ld a,(cursor_pos)
        inc a
        and %00000111
        jr z,test_sw
        ld a,(cursor_pos)
        inc a
        call getcur_value
        or a
        jr z,test_sw
        ld c,1
test_sw
        ld a,(cursor_pos)
        add a,7
        cp 64
        jr nc,test_s
        and %00000111
        cp %00000111
        jr z,test_s
        ld a,(cursor_pos)
        add a,7
        call getcur_value
        or a
        jr z,test_s
        ld c,1
test_s
        ld a,(cursor_pos)
        add a,8
        cp 64
        jr nc,test_se
        ld a,(cursor_pos)
        add a,8
        call getcur_value
        or a
        jr z,test_se
        ld c,1
test_se        
        ld a,(cursor_pos)
        add a,9
        cp 64
        ret nc
        and %00000111
        ret z
        ld a,(cursor_pos)
        add a,9
        call getcur_value
        or a
        ret z
        ld c,1
        ret




display_main_screen
        call make_screen_grid
        call show_mark_color
        ROM_CALL(DISP_GRAPH)
        call count_scores
        call disp_game_info
        ret


change_marks_next_cursor
;d = color of new mark
        ld a,(cursor_pos)
        ld b,a
get_changes_nw
        ld a,b
        sub 9
        ld b,a
        jr c,no_changes_nw
        and %00000111
        cp %00000111
        jr z,no_changes_nw
        ld a,b
        call getcur_value
        or a
        jr z,no_changes_nw
        cp d
        jr z,some_changes_nw
        cp unused
        call z,error_unvalid_square
        jr get_changes_nw
some_changes_nw
        ld a,(cursor_pos)
        ld c,a
        ld a,b
        add a,9
        ld b,a
        cp c
        jr z,changes_made_nw
        ld e,b
        ld b,d
        call putval_at_cursor
        ld b,e
        jr some_changes_nw
changes_made_nw
no_changes_nw

        ld a,(cursor_pos)
        ld b,a
get_changes_n
        ld a,b
        sub 8
        ld b,a
        jr c,no_changes_n
        ld a,b
        call getcur_value
        or a
        jr z,no_changes_n
        cp d
        jr z,some_changes_n
        cp unused
        call z,error_unvalid_square
        jr get_changes_n
some_changes_n
        ld a,(cursor_pos)
        ld c,a
        ld a,b
        add a,8
        ld b,a
        cp c
        jr z,changes_made_n
        ld e,b
        ld b,d
        call putval_at_cursor
        ld b,e
        jr some_changes_n
changes_made_n
no_changes_n

        ld a,(cursor_pos)
        ld b,a
get_changes_ne
        ld a,b
        sub 7
        ld b,a
        jr c,no_changes_ne
        and %00000111
        jr z,no_changes_ne
        ld a,b
        call getcur_value
        or a
        jr z,no_changes_ne
        cp d
        jr z,some_changes_ne
        cp unused
        call z,error_unvalid_square
        jr get_changes_ne
some_changes_ne
        ld a,(cursor_pos)
        ld c,a
        ld a,b
        add a,7
        ld b,a
        cp c
        jr z,changes_made_ne
        ld e,b
        ld b,d
        call putval_at_cursor
        ld b,e
        jr some_changes_ne
changes_made_ne
no_changes_ne


        ld a,(cursor_pos)
        ld b,a
get_changes_w
        ld a,b
        sub 1
        ld b,a
        jr c,no_changes_w
        and %00000111
        cp %00000111
        jr z,no_changes_w
        ld a,b
        call getcur_value
        or a
        jr z,no_changes_w
        cp d
        jr z,some_changes_w
        cp unused
        call z,error_unvalid_square
        jr get_changes_w
some_changes_w
        ld a,(cursor_pos)
        ld c,a
        ld a,b
        inc a
        ld b,a
        cp c
        jr z,changes_made_w
        ld e,b
        ld b,d
        call putval_at_cursor
        ld b,e
        jr some_changes_w
changes_made_w
no_changes_w

        ld a,(cursor_pos)
        ld b,a
get_changes_e
        ld a,b
        inc a
        ld b,a
        and %00000111
        jr z,no_changes_e
        ld a,b
        call getcur_value
        or a
        jr z,no_changes_e
        cp d
        jr z,some_changes_e
        cp unused
        call z,error_unvalid_square
        jr get_changes_e
some_changes_e
        ld a,(cursor_pos)
        ld c,a
        ld a,b
        dec a
        ld b,a
        cp c
        jr z,changes_made_e
        ld e,b
        ld b,d
        call putval_at_cursor
        ld b,e
        jr some_changes_e
changes_made_e
no_changes_e


        ld a,(cursor_pos)
        ld b,a
get_changes_sw
        ld a,b
        add a,7
        ld b,a
        cp 64
        jr nc,no_changes_sw
        and %00000111
        cp %00000111
        jr z,no_changes_sw
        ld a,b
        call getcur_value
        or a
        jr z,no_changes_sw
        cp d
        jr z,some_changes_sw
        cp unused
        call z,error_unvalid_square
        jr get_changes_sw
some_changes_sw
        ld a,(cursor_pos)
        ld c,a
        ld a,b
        sub 7
        ld b,a
        cp c
        jr z,changes_made_sw
        ld e,b
        ld b,d
        call putval_at_cursor
        ld b,e
        jr some_changes_sw
changes_made_sw
no_changes_sw

        ld a,(cursor_pos)
        ld b,a
get_changes_s
        ld a,b
        add a,8
        ld b,a
        cp 64
        jr nc,no_changes_s
        ld a,b
        call getcur_value
        or a
        jr z,no_changes_s
        cp d
        jr z,some_changes_s
        cp unused
        call z,error_unvalid_square
        jr get_changes_s
some_changes_s
        ld a,(cursor_pos)
        ld c,a
        ld a,b
        sub 8
        ld b,a
        cp c
        jr z,changes_made_s
        ld e,b
        ld b,d
        call putval_at_cursor
        ld b,e
        jr some_changes_s
changes_made_s
no_changes_s


        ld a,(cursor_pos)
        ld b,a
get_changes_se
        ld a,b
        add a,9
        ld b,a
        cp 64
        jr nc,no_changes_se
        and %00000111
        jr z,no_changes_se
        ld a,b
        call getcur_value
        or a
        jr z,no_changes_se
        cp d
        jr z,some_changes_se
        cp unused
        call z,error_unvalid_square
        jr get_changes_se
some_changes_se
        ld a,(cursor_pos)
        ld c,a
        ld a,b
        sub 9
        ld b,a
        cp c
        jr z,changes_made_se
        ld e,b
        ld b,d
        call putval_at_cursor
        ld b,e
        jr some_changes_se
changes_made_se
no_changes_se
        ret




putval_at_cursor
;a=position
;b=value
        push de
        push af
        ld c,%00111111
        and %0000011
        sla a
        jr z,no_shift_putval
shift_putval
        srl b
        rrc c
        dec a
        jr nz,shift_putval
no_shift_putval
        pop af
        ld hl,board_start
        sra a
        sra a
        ld d,0
        ld e,a
        add hl,de
        ld a,(hl)
        and c
        or b
        ld (hl),a
        pop de
        ret



inv_cursor
        push de
        ld a,(cursor_pos)
        and %00000111
        ld hl,GRAPH_MEM-96
        ld d,0
        ld e,a
        add hl,de
        ld a,(cursor_pos)
        call adjust_a_for_ypos
        ld de,96
get_cursor_offset
        add hl,de
        dec a
        jr nz,get_cursor_offset
        ld b,8
        ld de,12
invert_at_cursor
        ld a,(hl)
        cpl
        ld (hl),a
        add hl,de
        djnz invert_at_cursor
        call display_graph_at_cursor
        pop de
        ret

display_graph_at_cursor
        push bc
        ld a,(cursor_pos)
        and %00000111
        ld c,a
        ld a,(cursor_pos)
        and %11111000
        ld b,a
        ld e,8
        call display_part_of_graph_mem
        pop bc
        ret


display_part_of_graph_mem
;b=ypos
;c=xpos
;e=height of part
 ld hl,GRAPH_MEM-12
 ld a,b
 push de
 ld de,12
 inc b
get_offset_dpogm
 add hl,de
 djnz get_offset_dpogm
 add hl,bc
 pop de
 ld b,a
display_line_of_graph_mem
 ld a,b
 inc b
 or $80
 call $07F3
 out (P_LCD_CON),a
 ld a,$20
 or c
 call $07F3
 out (P_LCD_CON),a
 ld a,(hl)
 call $07F3
 out (P_LCD_DATA),a
 ld a,e
 ld de,12
 add hl,de
 ld e,a
 dec e
 jr nz,display_line_of_graph_mem
 ret





restore_game_query
        ld hl,string_restore_query
        call disp_text_at_hl
        call get_a_keystroke
        cp G_DEL
        ret nz
        call clear_board
        call color_query
        ret

get_a_keystroke
        call GET_KEY
        or a
        jr z,get_a_keystroke
        ret

color_query
        ROM_CALL(CLEARLCD)
        ld hl,string_color_query
        call disp_text_at_hl
        ld a,white
        ld (computer_color),a
        call get_a_keystroke
        cp G_DEL
        ret nz
        ld a,black
        ld (computer_color),a
        ret


clear_board
        ld hl,board_start
        ld bc,15
        ld de,board_start+1
        ld (hl),0
        ldir

        ld hl,board_start+6
        ld a,%00000001
        ld (hl),a
        ld hl,board_start+7
        ld a,%10000000
        ld (hl),a
        ld hl,board_start+8
        ld a,%00000010
        ld (hl),a
        ld hl,board_start+9
        ld a,%01000000
        ld (hl),a
        ret


count_scores
        ld a,0
        ld (blacks),a
        ld (whites),a
count_loop
        push af
        call getcur_value
        cp black
        jr nz,count_white
        ld a,(blacks)
        inc a
        ld (blacks),a
        jr counted_one_square
count_white
        cp white
        jr nz,counted_one_square
        ld a,(whites)
        inc a
        ld (whites),a
counted_one_square
        pop af        
        inc a
        cp 64
        jr c,count_loop
        ret



put_de_8_lines_at_a
        push af
        push de
        ld c,a
        and %00000111
        ld e,a
        ld d,0
        add hl,de
        ld a,c
        call adjust_a_for_ypos
        ld de,96
get_mark_offset
        add hl,de
        dec a
        jr nz,get_mark_offset
        pop de
        ld b,8
        call put_de_at_hl_b_times
        pop af
        ret



adjust_a_for_ypos
        and %11111000
        srl a
        srl a
        srl a
        inc a
        ret


disp_text_at_hl
        ld b,(hl)
        inc hl
        ld c,(hl)
        ld (CURSOR_X),bc
        inc hl
        ROM_CALL(D_ZM_STR)
        ret

disp_game_info
        ld hl,string_player
        call disp_text_at_hl
        ld hl,string_ti82
        call disp_text_at_hl

        ld a,(computer_color)
        cp black
        jr z,player_white
        ld a,(blacks)
        ld b,a
        ld a,(whites)
        ld c,a
        jr got_correct_number
player_white
        ld a,(blacks)
        ld c,a
        ld a,(whites)
        ld b,a
got_correct_number
        ld de,score_player_position
        ld (CURSOR_X),de
        ld a,b
        call display_decimal_byte
        ld a,c
        ld bc,score_ti82_position
        ld (CURSOR_X),bc
        call display_decimal_byte
        ret



show_mark_color
        ld a,(computer_color)
        ld de,sprite_black
        ld b,6
        cp black
        jr nz,computer_white_color
        ld hl,mark_info_position_ti82
        call put_de_at_hl_b_times
        ld hl,mark_info_position_player
        jr common_program_smc
computer_white_color
        ld hl,mark_info_position_player
        call put_de_at_hl_b_times
        ld hl,mark_info_position_ti82
common_program_smc
        ld de,sprite_white
        ld b,6
        call put_de_at_hl_b_times
        ret


make_screen_grid
        ld a,0
        ld (tempx),a
        ld (tempy),a
putsquareloop
        ld hl,GRAPH_MEM-12
        ld a,(tempx)
        ld e,a
        ld d,0
        add hl,de
        ld a,(tempy)
        inc a
        ld de,12
putsquareoffset_loop
        add hl,de
        dec a
        jr nz,putsquareoffset_loop

        ld a,(tempx)
        ld c,a
        ld a,(tempy)
        add a,c
        call getcur_value

        ld de,sprite_square
        cp black
        jr z,put_black_square
        or a
        jr z,got_correct_sprite
        ld de,sprite_white_mark
        jr got_correct_sprite
put_black_square
        ld de,sprite_black_mark
got_correct_sprite

        ld b,8
        call put_de_at_hl_b_times

        ld a,(tempx)
        inc a
        ld (tempx),a
        and %00000111
        jr nz,putsquareloop
        ld (tempx),a
        ld a,(tempy)
        add a,8
        ld (tempy),a
        cp 64
        jr c,putsquareloop
        ret


put_de_at_hl_b_times
        ld a,(de)
        ld (hl),a
        push de
        ld de,12
        add hl,de
        pop de
        inc de
        djnz put_de_at_hl_b_times
        ret

                          
get_best_position
        ld a,0
        ld (eval_position),a
        ld (best_position),a
        ld (best_wins),a
        ld b,255

find_first_free
        ld a,b
        inc a
        ld b,a
        cp 64
        jr nc,no_free_spaces    ;shouldn't happen since player starts
        call getcur_value
        or a
        jr nz,find_first_free
        ld a,b
        ld (best_position),a

        push de
        ld bc,$3545
        ld (CURSOR_X),bc
        ld a,(best_position)
        call display_decimal_byte
        ld bc,$1000
        call delay

        pop de


no_free_spaces
;shouldn't be jumped to

eval_loop

        push de
        ld bc,eval_pos_con
        ld (CURSOR_X),bc
        ld a,(eval_position)
        call display_decimal_byte
        pop de

        ld a,(eval_position)
        call getcur_value
        or a
        jr nz,curpos_not_available
        ld a,(eval_position)
        call evaluate_cur_pos   ;e = wins at position a

        ld a,(best_wins)
        cp e
        jr c,new_best_position
        jr nz,no_new_best_position

        ld a,(eval_position)
        or a
        jr z,cureval_is_corner
        cp 7
        jr z,cureval_is_corner
        cp 56
        jr z,cureval_is_corner
        cp 63
        jr z,cureval_is_corner

        ld a,(best_position)
        and %00000111
        jr z,no_new_best_position
        cp %00000111
        jr z,no_new_best_position
        ld a,(best_position)
        cp 8
        jr c,no_new_best_position
        cp 56
        jr nc,no_new_best_position

cureval_is_corner
new_best_position
        ld a,e
        ld (best_wins),a
        ld a,(eval_position)
        ld (best_position),a

        ld bc,eval_val_con
        ld (CURSOR_X),bc
        ld a,e
        push de
        call display_decimal_byte
        pop de
no_new_best_position
curpos_not_available
        ld a,(eval_position)
        inc a
        ld (eval_position),a
        cp 64
        jr c,eval_loop_help
        ret

eval_loop_help
        jp eval_loop

evaluate_cur_pos
;a=position
;d=colour, %01000000 = black , %10000000 = white
;Result: e = wins
;
        ld h,a
        ld e,0
        ld b,a
nw_neighbours
        ld a,b
        sub 9
        jr c,no_wins_nw
        ld b,a
        and %00000111
        cp %00000111
        jr z,no_wins_nw
        ld a,b
        call getcur_value
        or a
        jr z,no_wins_nw
        ld l,a
        cp d
        jr z,some_wins_nw
        ld a,l
        cp unused
        call z,error_unvalid_square
        inc e
        jr nw_neighbours
no_wins_nw
        ld e,0        
some_wins_nw

        ld a,h
        ld c,0
        ld b,a
n_neighbours
        ld a,b
        sub 8
        jr c,no_wins_n
        ld b,a
        call getcur_value
        or a
        jr z,no_wins_n
        ld l,a
        cp d
        jr z,some_wins_n
        ld a,l
        cp unused
        call z,error_unvalid_square
        inc c
        jr n_neighbours
some_wins_n
        ld a,c
        add a,e
        ld e,a
no_wins_n

        ld a,h
        ld c,0
        ld b,a
ne_neighbours
        ld a,b
        sub 7
        jr c,no_wins_ne
        ld b,a
        and %00000111
        jr z,no_wins_ne
        ld a,b
        call getcur_value
        or a
        jr z,no_wins_ne
        ld l,a
        cp d
        jr z,some_wins_ne
        ld a,l
        cp unused
        call z,error_unvalid_square
        inc c
        jr ne_neighbours
some_wins_ne
        ld a,c
        add a,e
        ld e,a
no_wins_ne

        ld a,h
        ld c,0
        ld b,a
w_neighbours
        ld a,b
        dec a
        ld b,a
        and %00000111
        cp %00000111
        jr z,no_wins_w
        ld a,b
        call getcur_value
        or a
        jr z,no_wins_w
        ld l,a
        cp d
        jr z,some_wins_w
        ld a,l
        cp unused
        call z,error_unvalid_square
        inc c
        jr w_neighbours
some_wins_w
        ld a,c
        add a,e
        ld e,a
no_wins_w

        ld a,h
        ld c,0
        ld b,a
e_neighbours
        ld a,b
        inc a
        ld b,a
        and %00000111
        jr z,no_wins_e
        ld a,b
        call getcur_value
        or a
        jr z,no_wins_e
        ld l,a
        cp d
        jr z,some_wins_e
        ld a,l
        cp unused
        call z,error_unvalid_square
        inc c
        jr e_neighbours
some_wins_e
        ld a,c
        add a,e
        ld e,a
no_wins_e

        ld a,h
        ld c,0
        ld b,a
sw_neighbours
        ld a,b
        add a,7
        cp 64
        jr nc,no_wins_sw
        ld b,a
        and %00000111
        cp %00000111
        jr z,no_wins_sw
        ld a,b
        call getcur_value
        or a
        jr z,no_wins_sw
        ld l,a
        cp d
        jr z,some_wins_sw
        ld a,l
        cp unused
        call z,error_unvalid_square
        inc c
        jr sw_neighbours
some_wins_sw
        ld a,c
        add a,e
        ld e,a
no_wins_sw

        ld a,h
        ld c,0
        ld b,a
s_neighbours
        ld a,b
        add a,8
        cp 64
        jr nc,no_wins_s
        ld b,a
        call getcur_value
        or a
        jr z,no_wins_s
        ld l,a
        cp d
        jr z,some_wins_s
        ld a,l
        cp unused
        call z,error_unvalid_square
        inc c
        jr s_neighbours
some_wins_s
        ld a,c
        add a,e
        ld e,a
no_wins_s

        ld a,h
        ld c,0
        ld b,a
se_neighbours
        ld a,b
        add a,9
        cp 64
        jr nc,no_wins_se
        ld b,a
        and %00000111
        jr z,no_wins_se
        ld a,b
        call getcur_value
        or a
        jr z,no_wins_se
        ld l,a
        cp d
        jr z,some_wins_se
        ld a,l
        cp unused
        call z,error_unvalid_square
        inc c
        jr se_neighbours
some_wins_se
        ld a,c
        add a,e
        ld e,a
no_wins_se
        ret




getcur_value
;a=position on board
        push bc
        push hl
        push af
        srl a
        srl a
        ld hl,board_start
        ld b,0
        ld c,a
        add hl,bc
        pop af
        and %00000011
        ld b,a
        ld a,(hl)
getcur_loop
        ld c,a
        ld a,b
        or a
        jr z,getcur_rotated
        ld a,c
        add a,a
        add a,a
        dec b
        jr getcur_loop
getcur_rotated
        ld a,c
        and %11000000
        pop hl
        pop bc
        ret



error_unvalid_square
        push bc
        push hl
        ld bc,$0000
        ld (CURSOR_POS),bc
        ld hl,string_error_square
        ROM_CALL(D_ZT_STR)
        pop hl
        pop bc
        ret


;*********** subroutine for displaying decimal numbers *******
display_decimal_byte
 push bc        
 ld b,255
ddb_get_hundred
 inc b
 sub 100
 jr nc,ddb_get_hundred
 add a,100
 call m_charput_of_b
 ld b,255
ddb_get_ten
 inc b
 sub 10
 jr nc,ddb_get_ten
 add a,10
 call m_charput_of_b
 ld b,a
 call m_charput_of_b
; add a,$30
; ROM_CALL(M_CHARPUT)
 pop bc        
 ret

m_charput_of_b
 push af
 ld a,b
 add a,$30
 ROM_CALL(M_CHARPUT)
 pop af
 ret

;*********** end: subroutine for displaying decimal numbers *******




delay:
 push af
count:
 ld a,b
 or c
 jr z,donecount
 dec bc
 jr count
donecount:
 pop af
 ret



computer_moves
        ld hl,string_scanning
        call disp_text_at_hl
        ld a,(computer_color)
        ld d,a
        call get_best_position
        ld a,(computer_color)
        ld b,a
        ld a,(best_position)
        ld (cursor_pos),a
        call putval_at_cursor
        call change_marks_next_cursor
        call display_main_screen
;        ld a,27
;        ld (cursor_pos),a
        call check_if_gameover
        ld a,c
        or a
        jr z,game_is_over
        call get_a_keystroke
        call inv_cursor
        jp keyloop

check_if_gameover
        ld b,255
        ld c,0
checkgo_loop
        ld a,b
        inc a
        ld b,a
        cp 64
        ret nc
        call getcur_value
        or a
        jr nz,checkgo_loop
        ld c,1
        ret

game_is_over
        ld a,(blacks)
        ld b,a
        ld a,(whites)
        cp b
        ld hl,string_black_won
        jr nc,white_won
        jr got_winner
white_won
        ld hl,string_white_won
got_winner
        call disp_text_at_hl
        ld a,255
        ld (best_position),a
        call get_a_keystroke
        ret



string_scanning
        .dw $4124
        .db "Scan...",0


sprite_square
        .db %11111111
        .db %10000001
        .db %10000001
        .db %10000001
        .db %10000001
        .db %10000001
        .db %10000001
        .db %11111111

sprite_black_mark
        .db %11111111
        .db %10011001
        .db %10111101
        .db %11111111
        .db %11111111
        .db %10111101
        .db %10011001
        .db %11111111

sprite_white_mark
        .db %11111111
        .db %10011001
        .db %10100101
        .db %11000011
        .db %11000011
        .db %10100101
        .db %10011001
        .db %11111111


sprite_black
        .db %00011000
        .db %00111100
        .db %01111110
        .db %01111110
        .db %00111100
        .db %00011000

sprite_white
        .db %00011000
        .db %00100100
        .db %01000010
        .db %01000010
        .db %00100100
        .db %00011000



board_start
        .db %10010110,%01011001
        .db %01010101,%01010101
        .db %01010101,%01010101
        .db %10010100,%01010110
        .db %01010101,%01010101
        .db %01010101,%01010101
        .db %10010101,%01010101
        .db %01010110,%01010110


string_error_square
        .db "Error: Board",0
string_player
        .dw $4200
        .db "Player:",0
string_ti82
        .dw $4212
        .db "TI82:",0

string_black_won
        .dw $1010
        .db "Black color won!",0
string_white_won
        .dw $1010
        .db "White color won!",0

string_restore_query
        .dw $0810
        .db "Press DEL to reset data",0
computer_color
        .db black
string_color_query
        .dw $0810
        .db "Press DEL to select white",0


        .end


